from __future__ import annotations
from typing import TYPE_CHECKING
from datetime import datetime

from sqlalchemy import (
    ForeignKey,
    String,
    DateTime,
    Boolean,
)
from sqlalchemy.orm import (
    Mapped,
    mapped_column,
    relationship,
)

from .Base import Base
from .mixins import UidMixin
if TYPE_CHECKING:
    from .BookcaseItem import BookcaseItem

class BookcaseItemBorrowingQueue(Base, UidMixin):
    username: Mapped[str] = mapped_column(String)
    entered_queue_time: Mapped[datetime] = mapped_column(DateTime, default=datetime.now())
    item_became_available_time: Mapped[datetime | None] = mapped_column(DateTime)
    expired = mapped_column(Boolean, default=False)

    fk_bookcase_item_uid: Mapped[int] = mapped_column(ForeignKey('BookcaseItem.uid'), index=True)

    item: Mapped[BookcaseItem] = relationship(back_populates='borrowing_queue')

    def __init__(
        self,
        username: str,
        item: BookcaseItem,
    ):
        self.username = username
        self.item = item
        self.entered_queue_time = datetime.now()